/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef BIT_GTKBUFFERTABLE_H
#define BIT_GTKBUFFERTABLE_H

#include <vector>
#include <map>

#include <gtkmm.h>

#include <bit/bit.h>

namespace bitGtk
  {

  /**
  @author Rick L. Vinyard, Jr.
  */
  class BufferTable : public Gtk::HBox
    {
    public:
      BufferTable();

      ~BufferTable();

      void set_record(bit::FieldBase::pointer record);

      void set_buffer(bit::RecordBuffer::pointer buffer);

      void set_bit_width(size_t w);

      void enable_tooltips(bool b=true);

      void enable_highlighter(bool b=true);
      void set_bg_color(std::string s);

      void add_color_set(std::string normal_color, std::string highlight_color="yellow");
      void clear_color_sets();

    protected:
      struct FieldBox
        {
          FieldBox() { }
          std::vector<Gtk::Label*> m_field_labels;
          std::vector<Gtk::EventBox*> m_field_event_boxes;
          std::vector<Gtk::Label*> m_value_labels;
          std::vector<Gtk::EventBox*> m_value_event_boxes;
          std::vector<Gtk::VBox*> m_highlight_boxes;
        };

      std::map<bit::FieldBase::pointer, FieldBox > m_fieldbox;

      bit::RecordBuffer::pointer m_buffer;
      bit::FieldBase::pointer m_record;
      std::list<std::pair<Gdk::Color, Gdk::Color> > m_colors;
      Gtk::Tooltips m_tooltips;
      bool m_highlighter_enabled;
      Gdk::Color m_bg_color;

      size_t m_display_bit_width;
      Gtk::Table* m_table;
      Gtk::EventBox m_tablebox;
      Gtk::EventBox m_tableboxouter;

      void on_record_changed();
      void on_field_value_changed(bit::FieldBuffer);

      void set_field_value( bit::FieldBase::pointer field );
      void load_all_field_values();
      void load_all_field_values( bit::FieldBase::pointer );

      void create_table();
      void setup_table();
      void create_fieldboxes(bit::FieldBase::pointer fb);
      void add_fieldbox(bit::FieldBase::pointer, unsigned left, unsigned right, unsigned top, unsigned bottom, Gdk::Color bg, Gdk::Color hl);

      bool on_enter(GdkEventCrossing*, bit::FieldBase::pointer);
      bool on_leave(GdkEventCrossing*, bit::FieldBase::pointer);

    };

}

#endif
